%% Exercise 3: Parametric and nonparametric (randomization) test of significance of
% autocorrelation using the Portmanteau test of Ljung-Box for simulated data.
n = 100; % Sample size
phiV = [0]'; % The coefficient vector of size (p+1) x 1 for an AR(p) generating process. 
          % x(t)=phi_0 + phi_1 x(t-1) + ... + phi_p1 x(t-p) + e_t,
          % e_t~WN(0,sigma^2)
          % phiV = [phi_0 phi_1 ... phi_p]'. 
          % If phiV=c then the system is x(t)= c + e_t (white noise with mean c). 
sigma = 1; % The standard deviation of the input white noise e_t 
noisedistr = 1; % if 1 then use normal distribution, if 2 use lognormal.
ntest = 100; % repititions of the test;
M = 100; % number of randomized samples.
tmax = 5;
alpha = 0.05;
disptest = 1;
toplot = 0;

nphi = length(phiV);
p = nphi-1;
pvalparV = NaN*ones(ntest,1);
pvalranV = NaN*ones(ntest,1);
QstatV = NaN*ones(ntest,1);
QrnkV = NaN*ones(ntest,1);
if disptest
    fprintf('Q statistic  par p-value    rank order   resampled p-value \n');
end
for itest=1:ntest
    %% Generate the time series  
    if noisedistr == 2
        xV = lognrnd(phiV,sigma,n,1);
    elseif noisedistr == 1
        if length(phiV)==1
            xV = phiV + sigma*randn(n,1);
        else
            xV = ARm(phiV,n,sigma);
        end
    end
    %% Parametric Postmanteau test
    [hV,pV,QV] = portmanteauLB(xV,tmax,alpha);
    pvalparV(itest)=pV(tmax);
    QstatV(itest)=QV(tmax);
    %% Randomized Portmanteau test
    % Compute randomized samples and make the test
    QsurV = NaN*ones(M+1,1);
    QsurV(1)=QstatV(itest);
    for im=1:M
        yV = xV(randperm(n)); % Random suffling of the time series
        [hV,pV,QV] = portmanteauLB(yV,tmax,alpha);
        QsurV(im+1)=QV(tmax);
    end 
    [oQsurV, iQsurV]=sort(QsurV); % Sort the M+1 Q-values
    QrnkV(itest) = find(iQsurV == 1); % and find the rank of the original
    pvalranV(itest) = 1-(QrnkV(itest)-0.326)/(M+1+0.348); 
    %% Plot the results of the randomized test
    if disptest
        if toplot
            figure(1)
            clf
            plot(QsurV(1)*[1 1],[0 2],'k','linewidth',1.5)
            hold on
            for im=1:M
                plot(QsurV(im+1)*[1 1],[0 1])
            end
            plot(QsurV(1)*[1 1],[0 2],'k','linewidth',1.5)
            xlabel('Q statistic'),
            legend('original','resampled','Location','Best')
            title(sprintf('Repition %d',itest))
        end
        fprintf('%d. \t %3.3f \t %1.3f \t %d \t %1.3f \n',itest,QstatV(itest),...
            pvalparV(itest),QrnkV(itest),pvalranV(itest));
        % pause;
    end
end   
%% Display the results of the parametric and nonparametric test
if ~disptest
    fprintf('\n');
    fprintf('Q statistic  par p-value    rank order   resampled p-value \n');
    for itest=1:ntest
        fprintf('%d. \t %3.3f \t %1.3f \t %d \t %1.3f \n',itest,QstatV(itest),...
            pvalparV(itest),QrnkV(itest),pvalranV(itest));
    end
end
rejpar = length(find(pvalparV < alpha));
rejran = length(find(pvalranV < alpha));
fprintf('Rejections at alpha=%1.3f: parametric=%d/%d   nonparametric=%d/%d \n',...
    alpha,rejpar,ntest,rejran,ntest);
