function xV = NARm(aV,m,n,sdnoise,x0V)
% xV = NARm(aV,n,sdnoise)
% Generate an NAR(m) time series of up to second degree of length 'n' 
% with Gaussian input noise.
% Note that if the order of NAR is m, then aV = [a_0 a_1 ... a_p]',
% where p=(m+1)(m+2)/2 and the form of NAR(m) is
% x(t) = a_0 + a_1 x(t-1) + ... + a_m x(t-m) + a_(m+1) x(t-1)^2 + 
%   + a_(m+2) x(t-1)x(t-2) + ... + a_p x(t-m)^2 
% sdnoise is the SD of the input noise (if left out then sdnoise=1)
% If x0V is given it reads the first m initial values.
if nargin==4
    x0V = sdnoise*randn(m,1);
elseif nargin==3
    x0V = sdnoise*randn(m,1);
    sdnoise=1;
end
ntrans = 100;
p = length(aV);
if p~=(m+1)*(m+2)/2
    error(sprintf('The coefficient vector size %d does not agree with the order %d',...
        p,m));
end
aV = aV(:);
wV = randn(n+ntrans,1) * sdnoise;
xV = NaN*ones(n+ntrans,1);
xV(1:m) = x0V(:);
for i=m+1:n+ntrans
    xV(i) = aV(1) + aV(2:m+1)'*xV(i-1:-1:i-m)+wV(i); % The linear part
    xxM = xV(i-1:-1:i-m)*xV(i-1:-1:i-m)';
    inow = m+1;
    for j=1:m
        xV(i) = xV(i) + aV(inow+1:inow+m-j+1)'*xxM(j:m,j);
        inow = inow+m-j+1;
    end
end
xV = xV(ntrans+1:n+ntrans);
