function xM = coupledhenonmaps(m,c,n,aV)
% xM = coupledhenonmaps(c,n,aV)
% COUPLEDHENONMAPS generates m time series from identical coupled Henon
% maps with coupling strength c. Each map is coupled to the next circularly
% (the last is coupled back to the first). 
% INPUTS
% - m   : the number of coupled maps (time series)
% - c   : the coupling strength 
% - n   : the time series length
% - aV  : the two parameters in a vector (if omitted the standard
%         parameters for chaos are chosen) 
% OUTPUTS
% - xM  : the n x m matrix of the generated time series
% Reference 
% Xunhe Yin  Yong Rena and Xiuming Shana
% Synchronization of discrete spatiotemporal chaos by using variable structure control
% Chaos, Solitons & Fractals, Volume 14, Issue 7, October 2002, Pages 1077-1082 

if nargin==3
    aV = [1.4 0.3];
end
if length(c)==1
    cV = c*ones(m,1);
else
    cV = c;
end

ntrans = 100;
x0limV = [-1 1 -0.3 0.3]'; 
xM = NaN*ones(n+ntrans,m);
tmpM = rand(2,m);
xM(1,:)=(x0limV(4)-tmpM(1,:))/(x0limV(4)-x0limV(3)); 
xM(2,:)=(x0limV(2)-tmpM(1,:))/(x0limV(2)-x0limV(1));
if m>2
    for t=3:n+ntrans
        xM(t,1)= aV(1) - (0.5*cV(1)*(xM(t-1,m)+xM(t-1,2)) + ...
                (1-cV(1))*xM(t-1,1))^2 + aV(2)*xM(t-2,1);    
        for im=2:m-1
            xM(t,im)= aV(1) - (0.5*cV(im)*(xM(t-1,im-1)+xM(t-1,im+1)) + ...
                (1-cV(im))*xM(t-1,im))^2 + aV(2)*xM(t-2,im);
        end
        xM(t,m)= aV(1) - (0.5*cV(m)*(xM(t-1,m-1)+xM(t-1,1)) + ...
                (1-cV(m))*xM(t-1,m))^2 + aV(2)*xM(t-2,m);    
    end
elseif m==2
    for t=3:n+ntrans
        xM(t,1)= aV(1) - xM(t-1,1)^2 + cV(2)*(xM(t-1,1)^2-xM(t-1,2)^2) + aV(2)*xM(t-2,1);
        xM(t,2)= aV(1) - xM(t-1,2)^2 + cV(1)*(xM(t-1,2)^2-xM(t-1,1)^2) + aV(2)*xM(t-2,2);    
    end
end
xM = xM(ntrans+1:n+ntrans,:);
