% Compute and plot cross-correlation of two world financial indices, 
% check stationarity 
clear all
close all
datdir = 'C:\MyFiles\Teach\Networks\Data\';
dattxt = 'WorldMarketsLong';
pridir = 'C:\MyFiles\Teach\Networks\Figures\';
n = 300;
indxV = [1 3]';
tmax = 10;
alpha = 0.01; % Significance level
toprint = 1;

zalpha = norminv(1-alpha/2);
txtM = str2mat('USA','Australia','UnitedKingdom','Germany','Greece',...
    'Malaysia','SouthAfrica','Croatia');
%symbV = str2mat('''.-k''','''.--k''','''.-.k''','''.:k''','''.-c''',...
%    '''.--c''','''.-.c''','''.:c''','''.-r''','''.--r''','''.-.r''','''.:r''');
symbV = str2mat('''-k''','''-r''');
nindx = length(indxV);
tittxt = sprintf('%s - %s',deblank(txtM(indxV(1),:)),deblank(txtM(indxV(2),:)));
pritxt = sprintf('%s%s',deblank(txtM(indxV(1),:)),deblank(txtM(indxV(2),:)));
tauV = [-tmax:tmax]';

%% Load the two time series and plot them
eval(['yM=load(''',datdir,dattxt,'.dat'');'])
yM = yM(end-n+1:end,indxV);
xM = diff(log(yM));
%% Cross-correlation for Indices
ccyV = crosscorr(yM(:,1),yM(:,2),tmax);

figure(1)
clf
paperfigure;
plot(tauV,ccyV,'.-')
hold on
plot([-tmax tmax],[0 0],'k')
plot([-tmax tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([-tmax tmax],-(zalpha/sqrt(n))*[1 1],'--c')
ax = axis;
axis([-tmax tmax ax(3) ax(4)])
xlabel('\tau')
ylabel('r_{XY}(\tau)')
title(sprintf('%s indices, Cross-correlation',tittxt));

if toprint 
    set(gcf,'PaperPosition',[0 0 20 10])
    eval(['print -djpeg90 ',pridir,pritxt,'indexcrosscor.jpg'])
end
%% Cross-correlation for Returns
ccxV = crosscorr(xM(:,1),xM(:,2),tmax);

figure(2)
clf
paperfigure;
plot(tauV,ccxV,'.-')
hold on
plot([-tmax tmax],[0 0],'k')
plot([-tmax tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([-tmax tmax],-(zalpha/sqrt(n))*[1 1],'--c')
ax = axis;
axis([-tmax tmax ax(3) ax(4)])
xlabel('\tau')
ylabel('r_{XY}(\tau)')
title(sprintf('%s returns, Cross-correlation',tittxt));

if toprint 
    set(gcf,'PaperPosition',[0 0 20 10])
    eval(['print -djpeg90 ',pridir,pritxt,'returnscrosscor.jpg'])
end
