% Compute and plot cross-correlation of time series generated by NVAR 
clear all
pridir = 'C:\MyFiles\Teach\Networks\Figures\';
pritxt = 'HenonC14C08n300';
n = 300;
tmax = 10;
alpha = 0.05; % Significance level
toprint = 1;
% phi0V = [0 0]';
% phi1M = [0.95 0; 0 0.85];
K = 2;
pfit = 10;
% aV = [0 4 -4]; 
% aV = [1 0 0.3 -1.4 0 0];
% aM = [aV; aV]; % parameters for the NAR of K variables 
cV = [0.14 0.08];
sdnoise = 0.0;
autax = [-0.5 1];
%croax = [-0.3 0.3];
croax = [-0.5 1];
mutax = [0 1.0];
cmuax = [0 0.3];

zalpha = norminv(1-alpha/2);
tauV = [-tmax:tmax]';
%% Generate the K time series 
rand('state',0);
randn('state',0);
yM = coupledhenonmaps(K,cV,n);
% yM = NaN*ones(n,K);
% for k=1:K
%     x0V = rand(m,1);
%     yM(:,k) = NARm(aM(k,:),m,n,sdnoise,x0V);
% end

%% Plot the two original time series
figure(1)
clf
paperfigure;
plot(yM(:,1),'.-k')
hold on
plot(yM(:,2),'.-r')
xlabel('t')
ylabel('x(t)')
legend('X','Y','Location','Best')
title('The time series X and Y')

%% Autocorrelation of the two original time series
acy1M = autocor(yM(:,1),1,tmax);
acy2M = autocor(yM(:,2),1,tmax);
figure(2)
clf
paperfigure;
plot(acy1M(2:tmax+1,1),acy1M(2:tmax+1,2),'.-k')
hold on
plot(acy2M(2:tmax+1,1),acy2M(2:tmax+1,2),'.-r')
plot([0 tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([0 tmax],-(zalpha/sqrt(n))*[1 1],'--c')
axis([0 tmax autax])
xlabel('\tau')
ylabel('r(\tau)')
legend('X','Y','Location','Best')
title('Autocorrelation of the two time series')
%% Cross-correlation of the two original time series
ccyV = crosscorr(yM(:,1),yM(:,2),tmax);
figure(3)
clf
paperfigure;
plot(tauV,ccyV,'.-')
hold on
plot([-tmax tmax],[0 0],'k')
plot([-tmax tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([-tmax tmax],-(zalpha/sqrt(n))*[1 1],'--c')
axis([-tmax tmax croax])
xlabel('\tau')
ylabel('r_{XY}(\tau)')
title('Cross-correlation of the original time series');

%% Fit AR model to each of the two time series
xM = NaN*ones(n,K);
for k=1:K
    th=ar(yM(:,k),pfit);
    ypreV = predict(th,yM(:,k),1);
    xM(:,k) = yM(:,k) - ypreV{1};
end

%% Plot the two prewhitened time series
figure(4)
clf
paperfigure;
plot(xM(:,1),'.-k')
hold on
plot(xM(:,2),'.-r')
xlabel('t')
ylabel('x(t)')
legend('X','Y','Location','Best')
title('The prewhitened time series X and Y')

%% Autocorrelation of the two prewhitened time series
acx1M = autocor(xM(:,1),1,tmax);
acx2M = autocor(xM(:,2),1,tmax);
figure(5)
clf
paperfigure;
plot(acx1M(2:tmax+1,1),acx1M(2:tmax+1,2),'.-k')
hold on
plot(acx2M(2:tmax+1,1),acx2M(2:tmax+1,2),'.-r')
plot([0 tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([0 tmax],-(zalpha/sqrt(n))*[1 1],'--c')
axis([0 tmax autax])
xlabel('\tau')
ylabel('r(\tau)')
legend('X','Y','Location','Best')
title('Autocorrelation of the prewhitened time series')
%% Cross-correlation of the prewhitened time series
ccxV = crosscorr(xM(:,1),xM(:,2),tmax);
figure(6)
clf
paperfigure;
plot(tauV,ccxV,'.-')
hold on
plot([-tmax tmax],[0 0],'k')
plot([-tmax tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([-tmax tmax],-(zalpha/sqrt(n))*[1 1],'--c')
axis([-tmax tmax croax])
xlabel('\tau')
ylabel('r_{XY}(\tau)')
title('Cross-correlation of the prewhitened time series');

%% Mutual information of the two original time series
muty1M = mutual(yM(:,1),tmax);
muty2M = mutual(yM(:,2),tmax);
figure(7)
clf
paperfigure;
plot(muty1M(1:end,1),muty1M(1:end,2),'.-k')
hold on
plot(muty2M(1:end,1),muty2M(1:end,2),'.-r')
plot([0 tmax],[0 0],'k')
axis([0 tmax mutax(1) mutax(2)])
legend('X','Y','Location','Best')
xlabel('\tau')
ylabel('I(\tau)')
title('Delayed mutual information of the original time series')

%% Cross-mutual information of the two original time series
mutyM = mutualxyts(yM(:,1),yM(:,2),tmax);

figure(8)
clf
paperfigure;
plot(mutyM(:,1),mutyM(:,2),'.-')
hold on
ax = axis;
axis([-tmax tmax ax(3) ax(4)])
xlabel('\tau')
ylabel('I_{XY}(\tau)')
title('Cross mutual information of the original time series');
axis([-tmax tmax cmuax(1) cmuax(2)])

%% Mutual information of the two prewhitened time series
mutx1M = mutual(xM(:,1),tmax);
mutx2M = mutual(xM(:,2),tmax);
figure(9)
clf
paperfigure;
plot(mutx1M(1:end,1),mutx1M(1:end,2),'.-k')
hold on
plot(mutx2M(1:end,1),mutx2M(1:end,2),'.-r')
plot([0 tmax],[0 0],'k')
axis([0 tmax mutax(1) mutax(2)])
legend('X','Y','Location','Best')
xlabel('\tau')
ylabel('I(\tau)')
title('Delayed mutual information of the prewhitened time series')

%% Cross-mutual information of the two prewhitened time series
mutxM = mutualxyts(xM(:,1),xM(:,2),tmax);

figure(10)
clf
paperfigure;
plot(mutxM(:,1),mutxM(:,2),'.-')
hold on
ax = axis;
axis([-tmax tmax ax(3) ax(4)])
xlabel('\tau')
ylabel('I_{XY}(\tau)')
title('Cross mutual information of the prewhitened time series');
axis([-tmax tmax cmuax(1) cmuax(2)])

if toprint
    figure(1)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'oriser.jpg'])
    figure(2)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'oriaut.jpg'])
    figure(3)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'oricro.jpg'])
    figure(4)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'resser.jpg'])
    figure(5)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'resaut.jpg'])
    figure(6)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'rescro.jpg'])
    figure(7)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'orimut.jpg'])
    figure(8)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'oricmu.jpg'])
    figure(9)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'resmut.jpg'])
    figure(10)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'rescmu.jpg'])
end
