% Compute and plot cross-correlation of time series generated by VAR of
% order 1 on K variables
clear all
pridir = 'C:\MyFiles\Teach\Networks\Figures\';
pritxt = 'VAR3causality';
savdir = 'C:\MyFiles\Teach\Networks\Results\';
savtauV = [0 1 2];
n = 300;
tmax = 10;
alpha = 0.05; % Significance level
toprint = 0;
tosave = 0;
K = 3;
p = 1;
pfit = 5;
A = zeros(K,K,p+1);
A(:,:,1) =eye(K);
% A(:,:,2) =[-0.95 0; 0 -0.85];
% A(:,:,2) =[-0.95 0; -0.5 -0.85];
% A(:,:,2) =[-1.2 0.5; -0.6 -0.3];
 A(:,:,2) =[-0.95 0.5 0.3; 0 -0.85 -0.3; 0 0 -0.9];
%A(:,:,2) =[-0.95 0.2 -0.3 0.4 -0.8; 0 -0.2 -0.3 -0.4 0.9; 0 0 -0.1 -0.1 0.8; ...
%    0 0 0 -0.8 -0.9; 0 0 0 0 0.8];
% A(:,:,2) = [-1.5 0.1;-0.2 1.5];
% A(:,:,3) = [0.7 -0.3;0.1 0.7];
autax = [-0.3 1];
%croax = [-0.3 0.3];
croax = [-1 1];

symbV = str2mat('''.-k''','''.-b''','''.-r''','''.-c''','''.-g''',...
    '''.--k''','''.--b''','''.--r''','''.--c''','''.--g''',...
    '''.-.k''','''.-.b''','''.-.r''','''.-.c''','''.-.g''');
zalpha = norminv(1-alpha/2);
%% Generate the K time series 
randn('state',0);
% yM = NaN*ones(n,K);
% for k=1:K
%     yM(:,k) = ARm(phi1V,n);
% yM(:,2) = ARm(phi2V,n);

Abig = A(:,:,2);
for ip=2:p
    Abig = [Abig A(:,:,ip+1)];
end
Aaug = [-Abig; eye((p-1)*K) zeros((p-1)*K,K)];
lambda  = eig(Aaug);
if any(abs(lambda) > 1)
    warning('The specified AR model is unstable.')
end

modS = idarx(A,[],1);
eM = iddata([],randn(n,K));
yS = sim(modS,eM);
yM = yS.OutputData;

%% Plot the K original time series
figure(1)
clf
paperfigure;
eval(['plot(yM(:,1),',symbV(1,:),')'])
hold on
legtxt = ['''X_1'','];
for k=2:K
    eval(['plot(yM(:,k),',symbV(k,:),')'])
    legtxt = [legtxt,'''X_',int2str(k),''','];
end
xlabel('t')
ylabel('x(t)')
eval(['legend(',legtxt,'''Location'',''Best'')'])
title('The original time series')
set(gca,'Box','On')
%% Autocorrelation of the original time series
acyM = NaN*ones(tmax,K);
for k=1:K
    acM = autocor(yM(:,k),1,tmax);
    acyM(:,k) = acM(2:tmax+1,2);
end
tauautV = [1:tmax]';
figure(2)
clf
paperfigure;
eval(['plot(tauautV,acyM(:,1),',symbV(1,:),')'])
hold on
legtxt = ['''X_1'','];
for k=2:K
    eval(['plot(tauautV,acyM(:,k),',symbV(k,:),')'])
    legtxt = [legtxt,'''X_',int2str(k),''','];
end
plot([0 tmax],[0 0],'k')
plot([0 tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([0 tmax],-(zalpha/sqrt(n))*[1 1],'--c')
axis([0 tmax autax])
xlabel('\tau')
ylabel('r(\tau)')
eval(['legend(',legtxt,'''Location'',''Best'')'])
title('Autocorrelation of the time series')
set(gca,'Box','On')
%% Cross-correlation of the original time series
taucroV = [-tmax:tmax]';
figure(3)
clf
paperfigure;
hold on
count = 0;
legtxt = '';
for ik=1:K-1
    for jk=ik+1:K
        count = count + 1;
        ccyV = crosscorr(yM(:,ik),yM(:,jk),tmax);
        eval(['plot(taucroV,ccyV,',symbV(count,:),')'])
        legtxt = [legtxt,'''X_{',int2str(ik),',t} X_{',int2str(jk),',t+\tau}'','];
    end
end
plot([-tmax tmax],[0 0],'k')
plot([-tmax tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([-tmax tmax],-(zalpha/sqrt(n))*[1 1],'--c')
axis([-tmax tmax croax])
xlabel('\tau')
ylabel('r_{XY}(\tau)')
eval(['legend(',legtxt,'''Location'',''Best'')'])
title('Cross-correlation of the original time series');
set(gca,'Box','On')
%% Fit AR model to each of the two time series
xM = NaN*ones(n,K);
for k=1:K
    th=ar(yM(:,k),pfit);
    ypreV = predict(th,yM(:,k),1);
    xM(:,k) = yM(:,k) - ypreV{1};
end

%% Plot the K prewhitened time series
figure(4)
clf
paperfigure;
eval(['plot(xM(:,1),',symbV(1,:),')'])
hold on
legtxt = ['''X_1'','];
for k=2:K
    eval(['plot(xM(:,k),',symbV(k,:),')'])
    legtxt = [legtxt,'''X_',int2str(k),''','];
end
xlabel('t')
ylabel('x(t)')
eval(['legend(',legtxt,'''Location'',''Best'')'])
title('The prewhitened time series')
set(gca,'Box','On')
%% Autocorrelation of the prewhitened time series
acxM = NaN*ones(tmax,K);
for k=1:K
    acM = autocor(xM(:,k),1,tmax);
    acxM(:,k) = acM(2:tmax+1,2);
end
tauautV = [1:tmax]';
figure(5)
clf
paperfigure;
eval(['plot(tauautV,acxM(:,1),',symbV(1,:),')'])
hold on
legtxt = ['''X_1'','];
for k=2:K
    eval(['plot(tauautV,acxM(:,k),',symbV(k,:),')'])
    legtxt = [legtxt,'''X_',int2str(k),''','];
end
plot([0 tmax],[0 0],'k')
plot([0 tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([0 tmax],-(zalpha/sqrt(n))*[1 1],'--c')
axis([0 tmax autax])
xlabel('\tau')
ylabel('r(\tau)')
eval(['legend(',legtxt,'''Location'',''Best'')'])
title('Autocorrelation of the prewhitened time series')
set(gca,'Box','On')
%% Cross-correlation of the prewhitened time series
taucroV = [-tmax:tmax]';
ccxT = NaN*ones(K,K,tmax+1);
figure(6)
clf
paperfigure;
hold on
count = 0;
legtxt = '';
for ik=1:K-1
    for jk=ik+1:K
        count = count + 1;
        ccxV = crosscorr(xM(:,ik),xM(:,jk),tmax);
        ccxT(ik,jk,:) = ccxV(tmax+1:2*tmax+1);
        ccxT(jk,ik,:) = ccxV(tmax+1:-1:1);
        eval(['plot(taucroV,ccxV,',symbV(count,:),')'])
        legtxt = [legtxt,'''X_{',int2str(ik),',t} X_{',int2str(jk),',t+\tau}'','];
    end
end
plot([-tmax tmax],[0 0],'k')
plot([-tmax tmax],(zalpha/sqrt(n))*[1 1],'--c')
plot([-tmax tmax],-(zalpha/sqrt(n))*[1 1],'--c')
axis([-tmax tmax croax])
xlabel('\tau')
ylabel('r_{XY}(\tau)')
eval(['legend(',legtxt,'''Location'',''Best'')'])
title('Cross-correlation of the prewhitened time series');
set(gca,'Box','On')

%% Save results on cross correlation
if tosave
    eval(['fid1=fopen(''',savdir,pritxt,'cro.txt'',''w'');'])
    eval(['fid2=fopen(''',savdir,pritxt,'croadj.txt'',''w'');'])
    nsavtau = length(savtauV);
    for itau=1:nsavtau
        tau = savtauV(itau);
        fprintf(fid1,'tau=%d \n',tau);
        fprintf(fid2,'tau=%d \n',tau);
        ccM = ccxT(:,:,tau+1);
        adjM = abs(ccM)> abs(zalpha/sqrt(n));
        for ik=1:K
            for jk=1:K
                if jk==1
                    if ik~=jk
                        fprintf(fid1,' %1.2f ',ccM(ik,jk));
                        fprintf(fid2,' %d ',adjM(ik,jk));
                    end
                else
                    if ik==jk
                        fprintf(fid1,' & ');
                        fprintf(fid2,' & ');
                    else
                        fprintf(fid1,' & %1.2f ',ccM(ik,jk));
                        fprintf(fid2,' & %d ',adjM(ik,jk));
                    end
                end
            end
            fprintf(fid1,'\\\\ \n');
            fprintf(fid2,'\\\\ \n');
        end
    end
    fclose(fid1);
    fclose(fid2);
end

%% Print figures
if toprint
    figure(1)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'oriser.jpg'])
    figure(2)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'oriaut.jpg'])
    figure(3)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'oricro.jpg'])
    figure(4)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'resser.jpg'])
    figure(5)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'resaut.jpg'])
    figure(6)
    set(gcf,'PaperPosition',[0 0 15 10])
    eval(['print -djpeg90 ',pridir,pritxt,'rescro.jpg'])
end
