function h = plotcolormap(meaM,nameM,tittxt,figno,showcolorbar,cax) 
% h = plotcolormap(meaM,nameM,tittxt,figno,showcolorbar,cax) 
% PLOTCOLORMAP makes a color map plot in the given figure 'figno' for the
% components of input matrix 'meaM', setting a title 'tittxt', using the
% names in 'nameM' (in string matrix form) as ticks for x- and y-axis. 
% If showcolorbar is 1 then show also the color bar. 
% If cax is nonempty then this should be a two component vector
% determinging the colored axis
% The figure handle is given to the output.

if nargin==5
    cax = [];
elseif nargin==4
    cax = [];
    showcolorbar = 0;
elseif nargin==3
    cax = [];
    showcolorbar = 0;
    figno = gcf+1;
elseif nargin==2
    cax = [];
    showcolorbar = 0;
    figno = gcf+1;
    tittxt = [];
elseif nargin==1
    cax = [];
    showcolorbar = 0;
    figno = gcf+1;
    tittxt = [];
    nameM = [];
end

K = size(meaM,1);
if isempty(nameM)
    nameM = str2mat('1');
    for ik=2:K
        nameM = str2mat(nameM,int2str(ik));
    end
end

h = figure(figno);
clf
[nrow,ncol]=size(meaM);
if nrow==ncol
    for i=1:nrow
        meaM(i,i)=NaN;
    end
end
plM = [[meaM ones(nrow,1)]; ones(1,ncol+1)];
pcolor(plM)
colormap(gray)
if ~isempty(cax)
    caxis([cax])
else
    mmin = min(min(meaM));
    mmax = max(max(meaM));
    % colormap(flipud(bone));
    if mmin<mmax
        caxis([mmin mmax])
    end
end
set(gca,'Box','On')
% set(gca,'Xtick',get(gca,'XTick')+0.5);
set(gca,'Xtick',[1:ncol]+0.5);
set(gca,'XTickLabel',nameM)
% rotateticklabel(gca,30);
% set(gca,'Ytick',get(gca,'YTick')+0.5);
set(gca,'Ytick',[1:nrow]+0.5);
set(gca,'YTickLabel',nameM)
title(tittxt)
if nrow==ncol
    for i=1:nrow
        text(i+0.3,i+0.5,'X','FontSize',32)
    end
end
if showcolorbar==1
    colorbar('location','southoutside')
elseif showcolorbar==2
    colorbar('location','eastoutside')
end
