% Parametric test of significance of the partial correlation coefficient
% for the world financial indices. 
clear all
datdir = 'C:\MyFiles\Teach\Networks\Data\';
dattxt = 'WorldMarkets';
pridir = 'C:\MyFiles\Teach\Networks\Figures\';
filename = 'C:\MyFiles\Teach\Networks\Data\WorldIndicesPartCorr.net';
n = 100;
usereturns = 0;
txtM = str2mat('USA','Australia','UnitedKingdom','Germany','Greece',...
    'Malaysia','SouthAfrica','Croatia');
alpha = 0.01; % Significance level
toplot = 0;
toprint = 0;
npart = 0; % 0->all the rest m-2 variables, else the given variables in the vector

symbV = str2mat('''.-k''','''.--k''','''.-.k''','''.:k''','''.-c''',...
    '''.--c''','''.-.c''','''.:c''','''.-r''','''.--r''','''.-.r''','''.:r''');
%% Load the time series and plot them
m = size(txtM,1);
eval(['xM=load(''',datdir,dattxt,'.dat'');'])
xM = xM(end-n:end,:);
if usereturns
    xM(1:n,:)=log(xM(2:n+1,:))-log(xM(1:n,:));
end
xM = xM(1:n,:);
%% Compute and plot correlation coefficients
count = 0;
rM = NaN*ones(m,m);
statM = NaN*ones(m,m); % upper -> part corr.coefs, lower-> p-values
adjM = NaN*ones(m,m); % adjacency matrix
for i=1:m-1
    for j=i+1:m
        fprintf('pair (%d,%d) ... \n',i,j);
        if npart==0
            xindV = setdiff([1:m],[i j]);
        else
            xindV = setdiff(npart,[i j]);
        end
        if isempty(xindV)
            e1V = xM(:,i);
            e2V = xM(:,j);
        else
            [tmp1,tmp2,e1V] = regress(xM(:,i),[ones(n,1) xM(:,xindV)]);
            [tmp1,tmp2,e2V] = regress(xM(:,j),[ones(n,1) xM(:,xindV)]);
        end
        tmpM = corrcoef(e1V,e2V);
        rM(i,j)=tmpM(1,2);
        count = count+1;
        statM(i,j) = sqrt(n-length(xindV)-3)*0.5*log((1+rM(i,j))/(1-rM(i,j)));
        statM(j,i) = 2*(1-normcdf(abs(statM(i,j))));
        statM(i,j)=rM(i,j);
        if statM(j,i)>alpha
            adjM(i,j)=0;
        else
            adjM(i,j)=1;
        end
        if toplot
            figure(count)
            clf
            paperfigure;
            plot(e1V,e2V,'.','Markersize',18);
            hold on
            xlabel(deblank(txtM(i,:)))
            ylabel(deblank(txtM(j,:)))
            title(sprintf('n=%d r_{%d,%d|K}=%1.3f',n,i,j,rM(i,j)))
            if toprint
                eval(['print -djpeg90 ',pridir,dattxt,'partrho',int2str(count),'.jpg'])
            end
        end
    end
end
statM;
adjM;
if ~isempty(filename)
    InputPajek(txtM,adjM,filename);
end
