% Script 3: cross correlations in greek stocks and networks
tau = 1;
alpha = 0.05;
K = 5;
nonparametric = 1; 
rthresh = 0.2;
M = 100;
maxtau = 20;
dattxt = 'GRstocks';

yM = load('d:\MyFiles\Teach\Networks\Data\stocks2003.dat');
[n,m]=size(yM);
rand('state',0)

iV = randperm(m);
% iV = [1:m];
yM = yM(:,iV(1:K));

% Read the names of the stocks
nameM = textread('d:\MyFiles\Teach\Networks\Data\stock_names.dat','%s');
nameM = nameM(iV(1:K),:);

% If NaN replace them with interpolated values for each time series
for i=1:K
    i1V = find(isnan(yM(:,i)));
    if ~isempty(i1V)
        iokV = setdiff([1:n]',i1V);
        yM(i1V,i) = interp1(iokV,yM(iokV,i),i1V,'spline');
    end
end

% Use log returns
xM = log(yM(2:n,:))-log(yM(1:n-1,:));

% For each pair compute the correlation matrix
ccM = NaN*ones(K,K);
p1M = zeros(K,K);
if tau==0
    % The correlation matrix is symmetric
    [ccM,p1M] = corrcoef(xM);
    p1M(1:K+1:K*K) = 0;
    if nonparametric
        % Randomized r values for the pair (y1,y2)
        p2M = zeros(K,K);
        ccsurT = NaN*ones(M,K,K);
        ccsurT(1,:,:) = ccM;
        for i=1:M
            zM = NaN*ones(n-1,K);
            for j=1:K
                randindx = ceil((n-maxtau)*rand);
                isurV = [randindx+1:n-1 1:randindx]';
                zM(:,j) = xM(isurV,j);
            end
            ccsurT(i+1,:,:) = corrcoef(zM);
        end
        for ik=1:K-1
            for jk=ik+1:K
                rxyV = squeeze(ccsurT(:,ik,jk));
            	p2M(ik,jk) = resampledpvalue(rxyV,1);
                p2M(jk,ik) = p2M(ik,jk);
            end
        end
    end
else
    % The correlation matrix is not symmetric
    for ik=1:K-1
        for jk=ik+1:K
            [tmpM,ptmpM] = corrcoef(xM(1:end-tau,ik),xM(1+tau:end,jk));
            ccM(ik,jk) = tmpM(1,2);
            p1M(ik,jk) = ptmpM(1,2);
            [tmpM,ptmpM] = corrcoef(xM(1:end-tau,jk),xM(1+tau:end,ik));
            ccM(jk,ik) = tmpM(1,2);
            p1M(jk,ik) = ptmpM(1,2);
        end
    end
    if nonparametric
        % Randomized r values for the pair (y1,y2)
        p2M = zeros(K,K);
        ccsurT = NaN*ones(M,K,K);
        ccsurT(1,:,:) = ccM;
        for i=1:M
            zM = NaN*ones(n-1,K);
            for j=1:K
                randindx = ceil((n-maxtau)*rand);
                isurV = [randindx+1:n-1 1:randindx]';
                zM(:,j) = xM(isurV,j);
            end
            % The correlation matrix is not symmetric
            for ik=1:K-1
                for jk=ik+1:K
                    tmpM = corrcoef(zM(1:end-tau,ik),zM(1+tau:end,jk));
                    ccsurT(i+1,ik,jk) = tmpM(1,2);
                    tmpM = corrcoef(zM(1:end-tau,jk),zM(1+tau:end,ik));
                    ccsurT(i+1,jk,ik) = tmpM(1,2);
                end
            end
        end
        for ik=1:K-1
            for jk=ik+1:K
                rxyV = squeeze(ccsurT(:,ik,jk));
            	p2M(ik,jk) = resampledpvalue(rxyV,1);
                rxyV = squeeze(ccsurT(:,jk,ik));
            	p2M(jk,ik) = resampledpvalue(rxyV,1);
            end
        end
    end
end    
tit1txt = sprintf('R_{XY}(%d)',tau);
% h1 = plotcolormap(ccM,nameM,tit1txt,1,1);
h1 = plotnetworktitle(ccM,[],nameM,tit1txt,1);

tit2txt = sprintf('parametric p-value, R_{XY}(%d)',tau);
% h2 = plotcolormap(p1M,nameM,tit2txt,2,1);
h2 = plotnetworktitle(p1M,[0 0.2],nameM,tit2txt,2);
tit3txt = sprintf('nonparametric p-value, R_{XY}(%d)',tau);
% h3 = plotcolormap(p2M,nameM,tit3txt,3,1);
h3 = plotnetworktitle(p2M,[0 0.2],nameM,tit3txt,3);

[iiV,jjV]=find(p1M < alpha);
adj1M = zeros(K,K);
for i=1:length(iiV)
    adj1M(iiV(i),jjV(i))=1;
end
tit4txt = sprintf('Adjacency parametric p(R_{XY}(%d)) < %1.2f',tau,alpha);
% h4 = plotcolormap(adj1M,nameM,tit4txt,4,1);
h4 = plotnetworktitle(adj1M,[0 1],nameM,tit4txt,4);

[iiV,jjV]=find(p2M < alpha);
adj2M = zeros(K,K);
for i=1:length(iiV)
    adj2M(iiV(i),jjV(i))=1;
end
tit5txt = sprintf('Adjacency nonparametric p(R_{XY}(%d)) < %1.2f',tau,alpha);
% h5 = plotcolormap(adj2M,nameM,tit5txt,5,1);
h5 = plotnetworktitle(adj2M,[0 1],nameM,tit5txt,5);

adjfdr1M = adjFDRmatrix(p1M,alpha,2);
tit6txt = sprintf('FDR (a=%1.3f), parametric p-value, R_{XY}(%d)',alpha,tau);
% h6 = plotcolormap(adjfdr1M,nameM,tit6txt,6,1);
h6 = plotnetworktitle(adjfdr1M,[0 1],nameM,tit6txt,6);

adjfdr2M = adjFDRmatrix(p2M,alpha,2);
tit7txt = sprintf('FDR (a=%1.3f), nonparametric p-value, R_{XY}(%d)',alpha,tau);
% h7 = plotcolormap(adjfdr2M,nameM,tit7txt,7,1);
h7 = plotnetworktitle(adjfdr2M,[0 1],nameM,tit7txt,7);

[iiV,jjV]=find(abs(ccM) > rthresh);
rthreshM = zeros(K,K);
for i=1:length(iiV)
    rthreshM(iiV(i),jjV(i))=1;
end
tit8txt = sprintf('Adjacency R_{XY}(%d) > %1.2f',tau,rthresh);
% h8 = plotcolormap(rthreshM,nameM,tit8txt,8,1);
h8 = plotnetworktitle(rthreshM,[0 1],nameM,tit8txt,8);

