% Parametric and Nonparametric (randomization) test of significance of the
% partial correlation coefficient for the microarray data.
datdir = 'C:\MyFiles\Teach\Networks\Data\';
dattxt = 'Ecoliv4Build6Palsson03';
pridir = 'C:\MyFiles\Teach\Networks\Figures\';
toplot = 1;
toprint = 1;
nsur = 1000;
npart = 0; % 0->all the rest m-2 variables, else the given variables in the vector

eval(['[xM,txtM,rawM]=xlsread(''',datdir,dattxt,'.xls'');'])
xM = xM';
geneC = txtM(2:end,1);
experC = txtM(1,2:end);
[n,m]=size(xM);
count = 0;
rM = NaN*ones(m,m);
statM = NaN*ones(m*(m-1)/2,2); % col 1,2->t-stat,p-value, 4
prnkM = NaN*ones(m*(m-1)/2,2); % col 1-> order, col 2-> p-value
for i=1:m-1
    for j=i+1:m
        fprintf('pair (%d,%d) ... \n',i,j);
        if npart==0
            xindV = setdiff([1:m],[i j]);
        else
            xindV = setdiff(npart,[i j]);
        end
        if isempty(xindV)
            e1V = xM(:,i);
            e2V = xM(:,j);
        else
            [tmp1,tmp2,e1V] = regress(xM(:,i),[ones(n,1) xM(:,xindV)]);
            [tmp1,tmp2,e2V] = regress(xM(:,j),[ones(n,1) xM(:,xindV)]);
        end
        tmpM = corrcoef(e1V,e2V);
        rM(i,j)=tmpM(1,2);
        count = count+1;
        statM(count,1) = sqrt(n-length(xindV)-3)*0.5*log((1+rM(i,j))/(1-rM(i,j)));
        statM(count,2) = 2*(1-normcdf(abs(statM(count,1))));
        rsurV = NaN*ones(nsur+1,1);
        rsurV(1)=rM(i,j);
        for isur=1:nsur
            yV = e2V(randperm(n));
            tmpM = corrcoef(e1V,yV);
            rsurV(isur+1)=tmpM(1,2);
            [orV, irV]=sort(rsurV);
            prnkM(count,1)=find(irV==1);
            if prnkM(count,1)>nsur/2
                prnkM(count,2) = 2*(1-(prnkM(count,1)-0.326)/(nsur+1+0.348));
            else
                prnkM(count,2) = 2*(prnkM(count,1)-0.326)/(nsur+1+0.348);
            end
        end
        if toplot
            figure((count-1)*2+1)
            clf
            paperfigure;
            plot(e1V,e2V,'.','Markersize',18);
            hold on
            xlabel(geneC{i})
            ylabel(geneC{j})
            title(sprintf('n=%d r_{%d,%d|%d}=%1.3f',n,i,j,xindV,rM(i,j)))
            if toprint
                eval(['print -djpeg90 ',pridir,dattxt,'partrho',int2str(count),'.jpg'])
            end
            figure(count*2)
            clf
            paperfigure;
            hist(rsurV(2:end));
            hold on
            ax = axis;
            plot(rsurV(1)*[1 1],[ax(3) ax(4)],'r')
            text(ax(1)+0.02*(ax(2)-ax(1)),ax(4)-0.1*(ax(4)-ax(3)),...
                sprintf('r_{%s,%s}',geneC{i},geneC{j}))
            if toprint
                eval(['print -djpeg90 ',pridir,dattxt,'partrrand',int2str(count),'.jpg'])
            end
        end
    end
end
[statM(:,1) prnkM(:,1)]
[statM(:,2) prnkM(:,2)]




